package edu.uoc.dtorrens.dtblib.kml;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

import android.util.Log;

public class KmlService 
{
	public static final String TAG = KmlService.class.getName();
	
	public static final int MODE_QUALSEVOL = 0;
	public static final int MODE_COTXE = 1;
	public static final int MODE_CAMINANT = 2;
	
	public static String inputStreamToString(InputStream in) throws IOException
	{
		StringBuffer out = new StringBuffer();
		byte[] b = new byte[4096];
		for(int n; (n = in.read(b)) != -1;)
		{
			out.append(new String(b, 0 , n));
		}
		return out.toString();
	}
	
	public static KmlDataSet calculaRuta(Double startLat, 
											Double startLng, 
											Double targetLat, 
											Double targetLng, 
											int mode) 
	{
	    return calculaRuta(startLat + "," + startLng, targetLat + "," + targetLng, mode);
	}
	
	public static KmlDataSet calculaRuta(String startCoords,
										 String targetCoords, 
										 int mode) 
	{
	    String urlModeCaminant = "http://maps.google.com/maps?" + "saddr=" + startCoords + "&daddr="
	            + targetCoords + "&sll=" + startCoords + "&dirflg=w&hl=en&ie=UTF8&z=14&output=kml";

	    Log.d(TAG, "urlPedestrianMode: "+urlModeCaminant);

	    String urlModeCotxe = "http://maps.google.com/maps?" + "saddr=" + startCoords + "&daddr="
	            + targetCoords + "&sll=" + startCoords + "&hl=en&ie=UTF8&z=14&output=kml";

	    Log.d(TAG, "urlCarMode: "+urlModeCotxe);

	    KmlDataSet kmlSet = null;
	    // pel mode qualsevol: tprimer provem mode caminant,si falla provem cotxe
	    if (mode==MODE_QUALSEVOL||mode==MODE_CAMINANT) kmlSet = KmlService.getKmlDataSet(urlModeCaminant);
	    if (mode==MODE_QUALSEVOL&&kmlSet==null||mode==MODE_COTXE) kmlSet = KmlService.getKmlDataSet(urlModeCotxe);
	    return kmlSet;
	}
	
	/**
	 * Retrieve navigation data set from either remote URL or String
	 * @param url
	 * @return navigation set
	 */
	public static KmlDataSet getKmlDataSet(String url) {

	    // urlString = "http://192.168.1.100:80/test.kml";
	    Log.d(TAG,"urlString -->> " + url);
	    KmlDataSet kmlDataSet = null;
	    try
	        {           
	        final URL aUrl = new URL(url);
	        final URLConnection conn = aUrl.openConnection();
	        conn.setReadTimeout(15 * 1000);  // timeout for reading the google maps data: 15 secs
	        conn.connect();

	        /* Get a SAXParser from the SAXPArserFactory. */
	        SAXParserFactory spf = SAXParserFactory.newInstance(); 
	        SAXParser sp = spf.newSAXParser(); 

	        /* Get the XMLReader of the SAXParser we created. */
	        XMLReader xr = sp.getXMLReader();

	        /* Create a new ContentHandler and apply it to the XML-Reader*/ 
	        KmlSaxHandler kmlSax2Handler = new KmlSaxHandler(); 
	        xr.setContentHandler(kmlSax2Handler); 

	        /* Parse the xml-data from our URL. */ 
	        xr.parse(new InputSource(aUrl.openStream()));

	        /* Our NavigationSaxHandler now provides the parsed data to us. */ 
	        kmlDataSet = kmlSax2Handler.getParsedData(); 

	        /* Set the result to be displayed in our GUI. */ 
	        Log.d(TAG,"kmlDataSet: "+kmlDataSet.toString());

	    } catch (Exception e) {
	        // Log.e(myapp.APP, "error with kml xml", e);
	        kmlDataSet = null;
	    }   

	    return kmlDataSet;
	}
}
