package edu.uoc.dtorrens.dtblib.kml;

import java.util.ArrayList;

import android.util.Log;

import com.google.android.gms.maps.model.Marker;

/**
 * 
 * @author Dani Torrens
 *
 */
public class Marcador 
{
	/**
	 */
	public final String TAG = Marcador.class.getName();
	public final static int TIPUS_INDEFINIT = -1;
	public final static int TIPUS_RUTA = 0;
	public final static int TIPUS_MARCA = 1;
	public final static int TIPUS_IMATGE = 2;
	public final static int TIPUS_NOTA = 3;
	public final static int TIPUS_ORIGEN = 4;
	public final static int TIPUS_DESTI = 5;
	
	/**
	 */
	private int id;
	/**
	 */
	private String titol;
	/**
	 */
	private String descripcio;
	/**
	 */
	private String adress;
	/**
	 */
	private String coordenades;
	/**
	 */
	private ArrayList<Punt> punts = new ArrayList<Punt>();
	/**
	 */
	private float lat;
	/**
	 */
	private float lon;
	/**
	 */
	private float alt;
	/**
	 */
	private int tipus;
	/**
	 */
	private String imatge;
	
	/**
	 */
	private Marker marker = null; //només per a maps v2
	
	//
	public Marcador()
	{
		titol = new String("");
		descripcio = new String("");
		tipus = TIPUS_INDEFINIT;
	}
	
	//---------------------
	/**
	 * @param t
	 */
	public void setTitol(String t)
	{
		titol = t;
	}
	
	/**
	 * @return
	 */
	public String getTitol()
	{
		return titol;
	}
	
	/**
	 * @param d
	 */
	public void setDescripcio(String d)
	{
		descripcio = d;
	}
	
	/**
	 * @return
	 */
	public String getDescripcio()
	{
		return descripcio;
	}
	
	public void setCoordenada(Double latitud, Double longitud)
	{
		setCoordenades(longitud.toString()
				+ ","+ latitud.toString());
	}
	
	public void appendCoordenada(Double latitud, Double longitud)
	{
		punts.add(new Punt(latitud, longitud) );
		
		if( punts.size() > 0 )
		{
			coordenades = coordenades + " \n " + longitud + "," + latitud;
		}else{
			setCoordenades( longitud + ", " + latitud );
		}
	}
	
	/**
	 * @param c
	 */
	public void setCoordenades(String c)
	{
		coordenades = c;
		
		if (coordenades != null && coordenades.trim().length() > 0) 
		{
	        String[] triplets = coordenades.trim().split(" ");
	        
	        for( int i = 0; i < triplets.length; i++ )
	        {
	        	String[] coords = triplets[i].split(",");
	        	if(coords.length == 2)
	        	{
	        		punts.add(new Punt( Double.parseDouble(coords[1]), Double.parseDouble(coords[0]) ) );
	        	}else if(coords.length == 3){
	        		punts.add(new Punt( Double.parseDouble(coords[1]), Double.parseDouble(coords[0]), Double.parseDouble(coords[2]) ) );
	        	}else{
	        		Log.e(TAG, "El punt " + i + " està incomplert :: " + coords.length);
	        	}
	        	
	        }
		}
		
	}
	
	/**
	 * @return
	 */
	public String getCoordenades()
	{
		return coordenades;
	}
	
	/**
	 * @param a
	 */
	public void setAdress(String a)
	{
		adress = a;
	}
	
	/**
	 * @return
	 */
	public String getAdress()
	{
		return adress;
	}
	
	public int getNumCoordenades()
	{
		String[] pairs = this.getCoordenades().trim().split(" ");
		
        Log.d(TAG, "coordenades.length=" + pairs.length);
        
        return pairs.length;
	}
	
	public int getNumPunts()
	{
		return punts.size();
	}
	
	public ArrayList<Punt> getPunts()
	{
		return punts;
	}
	
	/**
	 * @param mark
	 */
	public void setMarker(Marker mark)
	{
		marker = mark;
	}
	
	/**
	 * @return
	 */
	public Marker getMarker()
	{
		return marker;
	}
	
	/**
	 * @param t
	 */
	public void setTipus(int t)
	{
		tipus = t;
	}
	
	/**
	 * @return
	 */
	public int getTipus()
	{
		return tipus;
	}

	/**
	 * @param nom
	 */
	public void setImatge(String nom)
	{
		imatge = nom;
	}
	
	/**
	 * @return
	 */
	public String getImatge()
	{
		return imatge;
	}
	
	public boolean isRuta()
	{
		if( this.tipus == TIPUS_RUTA)
		{
			return true;
		}
		return false;
	}
}
