package edu.uoc.dtorrens.dtblib.localitzacio;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.location.Location;
import android.location.LocationManager;
import android.location.LocationProvider;
import android.os.Bundle;
import android.os.IBinder;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import edu.uoc.dtorrens.dtblib.R;


/**
 * 
 * @author Dani Torrens
 * Servei de localitzacio en un process al Background
 */
public class SimpleLocationService extends Service 
{
	private static final String TAG = SimpleLocationService.class.getName();
	public static final String BROADCAST_PASAPAS = "edu.uoc.dtorrens.dtblib.action.LOCATION_UPDATE";
	private static final int SERVEI_ID = 1;
	/**
	 */
	private LocationManager lm = null;
	private static final int INTERVAL = 1000;
	private static final float DISTANCIA = 10f;
	
	private void enviaEstat(PosicioListener pl)
	{
		Intent i = new Intent(SimpleLocationService.BROADCAST_PASAPAS);
		
		i.putExtra("posicio",pl.posicio);
		i.putExtra("provider", pl.provider);
		i.putExtra("estat", pl.estat);
		i.putExtra("actiu", pl.actiu);
		this.getApplicationContext().sendBroadcast(i);
	}
	
	private class PosicioListener implements android.location.LocationListener
	{
		protected Location posicio;
		protected int estat = LocationProvider.AVAILABLE;
		protected boolean actiu = true;
		protected String provider;
		
		public PosicioListener(String prv)
		{
			Log.d(TAG, "LocationListener " + prv);
			provider = prv;
			posicio = new Location(provider);	
			
		}
		
		
		@Override
		public void onLocationChanged(Location location) {
			Log.d(TAG, " Location Changed " + location);
			posicio.set(location);
			enviaEstat(this);
		}

		@Override
		public void onStatusChanged(String provider, int status, Bundle extras) {
			Log.d(TAG, " Status changed " + provider);
			estat = status;
			enviaEstat(this);
		}

		@Override
		public void onProviderEnabled(String provider) {
			Log.d(TAG, " Provider Enabled " + provider);
			actiu = true;
			enviaEstat(this);
		}

		@Override
		public void onProviderDisabled(String provider) {
			Log.d(TAG, " Provider Disabled " + provider);
			actiu = false;
			enviaEstat(this);
		}
	}
	
	/**
	 */
	PosicioListener[] locationListeners = new PosicioListener[] 
	{
			new PosicioListener(LocationManager.GPS_PROVIDER),
			new PosicioListener(LocationManager.NETWORK_PROVIDER)
	};
	
	@Override
	public IBinder onBind(Intent intent) {
		
		return null;
	}
	
	public int onStartCommand(Intent in, int flags, int startId)
	{
		Log.d(TAG, " Start Command ");
		super.onStartCommand(in, flags, startId);
		return START_STICKY;
	}

	public void onCreate()
	{
		Log.d(TAG, " on Create ");
		
		NotificationCompat.Builder b = new NotificationCompat.Builder(this)
		.setSmallIcon(R.drawable.ic_stat_running)
		.setContentTitle(getText(R.string.nota_servei_on_titol))
		.setContentText(getText(R.string.nota_servei_on_msg));


		startForeground(SERVEI_ID, b.build());
		
		iniciaLocationManager();
		try{
			lm.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, INTERVAL, DISTANCIA, locationListeners[1]);
		}catch( java.lang.SecurityException e )
		{
			Log.i(TAG, " ha fallat requestUpdate a Network manager ",e);
		}catch(IllegalArgumentException e)
		{
			Log.i(TAG, " no existeix Network provider ",e);
		}
		
		try{
			lm.requestLocationUpdates(LocationManager.GPS_PROVIDER, INTERVAL, DISTANCIA, locationListeners[0]);
		}catch( java.lang.SecurityException e )
		{
			Log.i(TAG, " ha fallat requestUpdate a GPS ",e);
		}catch(IllegalArgumentException e)
		{
			Log.i(TAG, " no existeix GPS provider ",e);
		}
	}
	
	public void onDestroy()
	{
		Log.d(TAG, "  DESTROY DESTROY DESTROY ");
		super.onDestroy();
		if( lm != null )
		{
			for( int i = 0; i < locationListeners.length; i++ )
			{
				try
				{
					lm.removeUpdates(locationListeners[i]);
				}catch(Exception e)
				{
					Log.i(TAG, "Error eliminant updates de location listener", e);
				}
			}
		}
	}
	
	private void iniciaLocationManager()
	{
		Log.d(TAG, " Iniciam location manager");
		if( lm == null )
		{
			lm = (LocationManager) getApplicationContext().getSystemService(Context.LOCATION_SERVICE);
		}
	}
}
