package edu.uoc.dtorrens.dtblib.mapes;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

import android.app.AlertDialog;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.Point;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnCameraChangeListener;
import com.google.android.gms.maps.GoogleMap.OnMapLongClickListener;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;
import com.google.android.maps.GeoPoint;

import edu.uoc.dtorrens.dtblib.R;
import edu.uoc.dtorrens.dtblib.dialeg.DialegListenerActivity;
import edu.uoc.dtorrens.dtblib.kml.KmlDataSet;
import edu.uoc.dtorrens.dtblib.kml.KmlHandler;
import edu.uoc.dtorrens.dtblib.kml.Marcador;
import edu.uoc.dtorrens.dtblib.kml.Punt;
import edu.uoc.dtorrens.dtblib.localitzacio.LocationUtils;
import edu.uoc.dtorrens.dtblib.localitzacio.MapsDosLocation;
import edu.uoc.dtorrens.dtblib.storage.StorageHelper;
import edu.uoc.dtorrens.utils.ScreenUtils;

public abstract class AbstractBaseMap extends DialegListenerActivity implements MapaInterface
{
	/**
	 */
	private final String TAG = AbstractBaseMap.class.getName();
	
	/**
	 */
	protected Context context;
	
	/**
	 */
	protected GoogleMap mapa;
	/**
	 */
	protected ArrayList<Marcador> marcadors = new ArrayList<Marcador>();
	/**
	 */
	protected KmlDataSet objKml = null;
	/**
	 */
	protected boolean firstChange = true;
	/**
	 */
	protected boolean boundsFound = false;
	/**
	 */
	protected double lat1;			// per enquadrar la vista

	/**
	 */
	protected double lat2;

	/**
	 */
	protected double lon1;

	/**
	 */
	protected double lon2;
	
	/**
	 */
	protected Location currentLocation = null;
	/**
	 */
	protected MapsDosLocation mapLocation = new MapsDosLocation(this);
	
	/**
	 */
	protected int imageCount = 0;
	
	/**
	 */
	protected boolean gravant = false;
	
	//new location v2
	/**
	 */
	StorageHelper storage;
	
	/**
	 */
	protected ImageView iv;
	/**
	 */
	protected TextView itv;
	/**
	 */
	protected TextView tv;
	
	abstract public void onDialegReturn(Object[] result, int code) ;
	
	//------ Utilitzats 
	public void saveKml(String fileName, String dir)
	{
		StringBuffer buffer = new StringBuffer();
		//creem el text
		buffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n");
		buffer.append("<Document>\n");
		buffer.append("\t<name>" + objKml.getTitol() + "</name>\n");
		buffer.append("\t<description>" + objKml.getDescripcio() + "</description>\n");
		
		// els punts
		for( Iterator<Marcador> it = this.objKml.getMarcadors().iterator(); it.hasNext(); )
		{
			Marcador marc = it.next();
			buffer.append("\t\t<Placemark>\n");
			buffer.append("\t\t\t<name>" + marc.getTitol() + "</name>\n");
			buffer.append("\t\t\t<description>" + marc.getDescripcio() + "</description>\n");
			
			buffer.append("\t\t\t<Point>\n\t\t\t\t<coordinates>" + marc.getCoordenades() + "</coordinates>\n\t\t\t</Point>\n");
			
			if( marc.getTipus() == Marcador.TIPUS_IMATGE )
			{
				buffer.append("\t\t\t<ExtendedData>\n\t\t\t\t<Data name=\"imatge\"><value>" + marc.getImatge() + "</value></Data>\n\t\t\t</ExtendedData>\n");
			}	
			
			
			buffer.append("\t\t</Placemark>\n");
		}
		
		//la ruta
		Marcador ruta = objKml.getMarcadorRuta();
		if( ruta != null )
		{
			buffer.append("\t\t<Placemark>\n");
			buffer.append("\t\t\t<name>" + ruta.getTitol() + "</name>\n");
			buffer.append("\t\t\t<description>" + ruta.getDescripcio() + "</description>\n");
			
			buffer.append("\t\t\t<LineString>\n\t\t\t\t<coordinates>" + ruta.getCoordenades() + "</coordinates>\n\t\t\t</LineString>\n");
			
			buffer.append("\t\t</Placemark>\n");
		}
		
		buffer.append("</Document>\n</kml>\n");
		
		File file = new File(dir, fileName);
		try {
			FileWriter out = new FileWriter(file);
			out.write(buffer.toString());
			out.flush();
			out.close();
		} catch (IOException e) {
			Log.e(TAG, "ERROR FileWriter :: " + e.getMessage());
			e.printStackTrace();
		}
	}
	
	//-----------------------------------------
	public void onResume()
	{
		super.onResume();
		setUpMapIfNeeded();
		
		mapLocation.getLocationManager().requestLocationUpdates(0L, 0.0f, mapLocation.getCriteria(), mapLocation, null);
	    mapa.setLocationSource(mapLocation);
	}
	public void onPause()
	{
		mapa.setLocationSource(null);
	    mapLocation.getLocationManager().removeUpdates(mapLocation);
	    
		super.onPause();
	}
	
	public void onDestroy()
	{
		Log.i(TAG,"ON DESTROY");
		cleanObjKml();
		super.onDestroy();
	}
	
	public void onBackPressed()
	{
		if(iv.getVisibility() == View.GONE)
		{
			super.onBackPressed();
		}else{
			iv.setVisibility(View.GONE);
			itv.setVisibility(View.GONE);
		}
	}
	
	public void init()
	{
		setUpMapIfNeeded();
	}
	
	public void cleanObjKml()
	{
		synchronized(context)
		{
			Log.i(TAG,"CLEAN");
			objKml = null;
			this.imageCount = 0;
			storage.currentPrivateToRoot();
			storage.createPrivateSubdir(StorageHelper.tempDir);
			storage.borraRecursiu(new File(storage.getPrivateCurrentDir()));
			
		}
	}
	
	protected void setUpMapIfNeeded() {
        if (mapa == null) {
            mapa = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map))
                    .getMap();
            if (mapa != null) {
            	setUpMap();                
            }
        }
    }
	
	 protected void setUpMap() {
		 	mapa.getUiSettings().setZoomControlsEnabled(true);	   
		 	mapa.getUiSettings().setCompassEnabled(true);
		 	mapa.getUiSettings().setMyLocationButtonEnabled(true);
		 	mapa.setMyLocationEnabled(true);
		 	
		 	mapLocation.getLocationManager().requestLocationUpdates(0L, 0.0f, mapLocation.getCriteria(), mapLocation, null);
		    mapa.setLocationSource(mapLocation);
		 	
		 	currentLocation = mapa.getMyLocation();
		 	
		 	if( currentLocation == null )
		 	{
		 		Log.i(TAG,"START Current location == NULL");
		 		mapLocation.forceLastKnownLocation();
		 	}
		 	
		 	//per depurar
		 	mapa.setOnMapLongClickListener(new OnMapLongClickListener(){

				@Override
				public void onMapLongClick(LatLng arg0) 
				{
					Log.i(TAG,"LONG PRESS ON MAP");
				}});
		 	
		 	mapa.setOnMarkerClickListener(new OnMarkerClickListener(){

				@Override
				public boolean onMarkerClick(Marker marker) {
					
					
					for( Iterator<Marcador> it = marcadors.iterator(); it.hasNext();  )
					{
						Marcador marc = it.next();
						
						if( marker.equals(marc.getMarker()) )
						{
							Log.i(TAG, "He trobat el marcador");
							if(marc.getTipus() == Marcador.TIPUS_IMATGE)
							{
								Log.i(TAG, " i és imatge");
								Bitmap bmp = BitmapFactory.decodeFile(storage.getPrivateCurrentDir() + "/" + marc.getImatge());
								iv.setImageBitmap(bmp);
								iv.setVisibility(View.VISIBLE);
								itv.setVisibility(View.VISIBLE);
								
							}else{
								Log.i(TAG, "No és imatge");
								AlertDialog.Builder dialog = new AlertDialog.Builder(context);
						    	dialog.setTitle(marc.getTitol() );
						    	dialog.setMessage(marc.getDescripcio());
						    	dialog.create();
						    	dialog.show();
							}
								
					    	return true;
						}
					}
					
					return false;
				}
		 		
		 	});
		 	
		 	mapa.setOnCameraChangeListener(new OnCameraChangeListener(){

				@Override
				public void onCameraChange(CameraPosition cpos) {
					Log.e(TAG, "Camera CHANGED !! ZOOM:" + cpos.zoom + " -- " + cpos.target.latitude + " : " + cpos.target.longitude);
					if(firstChange)
					{
						if(boundsFound)
						{
							Log.i(TAG, "BOUNDS FOUND");
							
							Point mida = ScreenUtils.screenSize(context);
							
							int ref = (mida.x>mida.y)? mida.y : mida.x;
							
							int padding = Double.valueOf( ref*0.15 ).intValue();
							
							mapa.animateCamera( CameraUpdateFactory.newLatLngBounds(  
									new LatLngBounds( 
											new LatLng( lat1, lon1  ),
											new LatLng( lat2, lon2  )
											),
									   padding ),
									   new GoogleMap.CancelableCallback() {
										
										@Override
										public void onFinish() {
											firstChange = false;
										}
										
										@Override
										public void onCancel() {
											firstChange = false;
										}
									});
						}else{
							Log.i(TAG, "BOUNDS NOT FOUND");
							CameraUpdate cu;
							if(currentLocation != null)
							{
								 cu = CameraUpdateFactory.newLatLngZoom(new LatLng( currentLocation.getLatitude(), currentLocation.getLongitude() ) ,
																					19);
							}else{
								 cu = CameraUpdateFactory.zoomTo(20);
							}
								mapa.animateCamera(cu,new GoogleMap.CancelableCallback() {
							
								@Override
								public void onFinish() {
									firstChange = false;
								}
								
								@Override
								public void onCancel() {
									firstChange = false;
								}
							});
						}
						
					}
				}});
	        
	    }
	 public void zoomFit()
		{
			if(firstChange)
			{
				if(boundsFound)
				{
					Log.i(TAG, "BOUNDS FOUND");
					
					Point mida = ScreenUtils.screenSize(context);
					
					int ref = (mida.x>mida.y)? mida.y : mida.x;
					
					int padding = Double.valueOf( ref*0.15 ).intValue();
					
					mapa.animateCamera( CameraUpdateFactory.newLatLngBounds(  
							new LatLngBounds( 
									new LatLng( lat1, lon1  ),
									new LatLng( lat2, lon2  )
									),
							   padding ),
							   new GoogleMap.CancelableCallback() {
								
								@Override
								public void onFinish() {
									// TODO Auto-generated method stub
									firstChange = false;
								}
								
								@Override
								public void onCancel() {
									// TODO Auto-generated method stub
									firstChange = false;
								}
							});
				}else{
					Log.i(TAG, "BOUNDS NOT FOUND");
					mapa.animateCamera(CameraUpdateFactory.zoomTo(14),new GoogleMap.CancelableCallback() {
						
						@Override
						public void onFinish() {
							// TODO Auto-generated method stub
							firstChange = false;
						}
						
						@Override
						public void onCancel() {
							// TODO Auto-generated method stub
							firstChange = false;
						}
					});
				}
				//firstChange = false;
			}
		}
	 
	public void onLocationChangeCallback(Location location, MapsDosLocation mapLOcation)
	{
		Log.i(TAG,"Location CALLBACK");
		
		if( LocationUtils.isBetterLocation(location, currentLocation) )
		{
			currentLocation= new Location(location);
		}
		
		String msg = "";
		if( objKml != null )
		{
			if( objKml.getTitol().length() > 0)
			{
				msg += objKml.getTitol().toUpperCase() + "\n";
			}
			
			if( objKml.getDescripcio().length() > 0)
			{
				msg += objKml.getDescripcio() + "\n\n";
			}
		}
		
		if(currentLocation != null )
		{
			
			msg += getResources().getString( R.string.trobada_posicio);
			
		}
		
		tv.setText( msg  );
		
    	//he d'esperar a que el mapa existeixi
    	if(!firstChange)
    	{
    		if(gravant)
    		{
    			objKml.getMarcadorRuta().appendCoordenada(location.getLatitude(), location.getLongitude());
    		}
    		
	    	mapa.clear();
	    	if( objKml != null )
	    	{
	    		drawKmlDataSet(objKml);
	    	}
	    	//centrar(location.getLatitude(), location.getLongitude());
	    	
    	}
	}
	 
	@Override
	public void centrar(int ilat, int ilon) 
	{
		centrar( Double.valueOf( ilat )/1E6, Double.valueOf( ilon )/1E6  );
	}

	@Override
	public void centrar(double dlat, double dlon) {
		mapa.moveCamera(CameraUpdateFactory.newLatLng(  new LatLng( dlat, dlon ) ) );
	}

	@Override
	public void centrar(GeoPoint centre) 
	{
		centrar( centre.getLatitudeE6(), centre.getLongitudeE6() );
	}

	@Override
	public void addMarcadorOverlay(Marcador marc) 
	{
		addMarcadorOverlay(marc, ALTRES);
	}
	
	public void addMarcadorOverlay(Marcador marc, int tipus) 
	{
		synchronized(context)
		{
			Log.i(TAG,"Add Marcador Overlay");
			Iterator<Punt> it = marc.getPunts().iterator();
			if( it.hasNext() )
			{
				float hue;
				
				if( tipus == DESTI )
				{
					Log.i(TAG,"COLOR DESTI");
					hue = BitmapDescriptorFactory.HUE_GREEN;
				}else if(tipus == ORIGEN){
					Log.i(TAG,"COLOR ORIGEN");
					hue = BitmapDescriptorFactory.HUE_RED;
				}else{
					Log.i(TAG,"COLOR ALTRES");
					hue = BitmapDescriptorFactory.HUE_ORANGE;
				}
			
				BitmapDescriptor bmp;
				if( marc.getTipus() ==  Marcador.TIPUS_IMATGE )
				{
					Drawable d = getResources().getDrawable(R.drawable.ic_marker_cam_orange);
					
					bmp = BitmapDescriptorFactory.fromBitmap( ((BitmapDrawable)d).getBitmap());
				}else if( marc.getTipus() ==  Marcador.TIPUS_NOTA )
				{
					hue = BitmapDescriptorFactory.HUE_YELLOW;
					bmp = BitmapDescriptorFactory.defaultMarker(hue);
				}else{
					bmp = BitmapDescriptorFactory.defaultMarker(hue);
				}
				
				Punt p = it.next();
				Marker marker = mapa.addMarker(new MarkerOptions()
			       .position(new LatLng( p.getLatitud(), p.getLongitud() ) )
			       .title(marc.getTitol())
			       .icon(bmp)
			       .snippet(marc.getDescripcio()));
				marc.setMarker(marker);
				marcadors.add(marc);
			}
		}
	}
	
	public void drawPath(KmlDataSet ds, int color, GoogleMap mapa)
	{
		Log.i(TAG,"Draw Path");
		if(ds != null)
		{
			Marcador ruta = ds.getMarcadorRuta();
			if( ruta != null && ruta.getPunts().size()>0)
			{
				PolylineOptions polyOptions = new PolylineOptions();
				
				for( Iterator<Punt> it = ruta.getPunts().iterator(); it.hasNext(); )
				{
					Punt p = it.next();
					polyOptions.add( new LatLng( p.getLatitud(), p.getLongitud()  ) );
					
				}
				polyOptions.color(color);
				mapa.addPolyline(polyOptions);
			}
		}
	}
	
	public void drawKmlDataSet(KmlDataSet ds)
	{
		Log.i(TAG,"Draw DataSet");
		//if(ds == null)
		//	return;
		
		// draw path
        //drawPath(ds, Color.parseColor("#add331"), mapa );
        drawPath(ds, Color.parseColor("#0000ff"), mapa );

        
        
        
        //pintem tots els punts
        Marcador marcOrigen = null;
        Marcador marcFinal = null;
        int count = 1;
        int mida = ds.getMarcadors().size();
        marcadors.clear();
		for( Iterator<Marcador> it = ds.getMarcadors().iterator(); it.hasNext(); )
        {
        	Marcador marc = it.next();
        	
        	if(marcOrigen == null)
        	{
        		marcOrigen = marc;
        		addMarcadorOverlay(marc, ORIGEN);
        	}else if(count == mida) {
        		marcFinal = marc;
        		addMarcadorOverlay(marc, DESTI);
        	}else{
        		addMarcadorOverlay(marc, ALTRES);
        	}
        	
        	count++;
        }
        
        if( marcOrigen != null && marcFinal != null )
        {
        	Punt pOrigen = null;
        	Punt pFinal = null;
        	
        	Iterator<Punt> itOrigen = marcOrigen.getPunts().iterator();
        	if(itOrigen.hasNext())
        	{
        		pOrigen = itOrigen.next();
        	}
        	
        	Iterator<Punt> itFinal = marcFinal.getPunts().iterator();
        	if(itFinal.hasNext())
        	{
        		pFinal = itFinal.next();
        	}
        	
        	if( pOrigen != null && pFinal != null)
        	{
        		Log.i(TAG, "ZOOM TO POINTS");
        		
        		
        		
        		if( pOrigen.getLatitud() < pFinal.getLatitud() )
        		{
        			lat1 = pOrigen.getLatitud();
        			lat2 = pFinal.getLatitud();
        		}else{
        			lat2 = pOrigen.getLatitud();
        			lat1 = pFinal.getLatitud();
        		}
        		
        		if( pOrigen.getLongitud() < pFinal.getLongitud() )
        		{
        			lon1 = pOrigen.getLongitud();
        			lon2 = pFinal.getLongitud();
        		}else{
        			lon2 = pOrigen.getLongitud();
        			lon1 = pFinal.getLongitud();
        		}
        		
        		if( currentLocation != null )
        		{
        			Log.i(TAG, "Current Location també juga a centrar");
        			if( currentLocation.getLatitude() < lat1 ) lat1 = currentLocation.getLatitude();
        			if( currentLocation.getLatitude() > lat2 ) lat2 = currentLocation.getLatitude();
        			if( currentLocation.getLongitude() < lon1 ) lon1 = currentLocation.getLongitude();
        			if( currentLocation.getLongitude() > lon2 ) lon2 = currentLocation.getLongitude();
        		}
        		
        		boundsFound = true;
            	
        	}
        }
        this.zoomFit();
	}
	
	@Override
	public void drawKml(InputStream in) {
		try{
        	
            KmlHandler kmlHandler = new KmlHandler();
            //get from file
           
            kmlHandler.parse(in);
            
            // get the results - should be a fully populated RSSFeed instance, or null on error
            KmlDataSet ds = kmlHandler.getKmlDataSet();

            //temp pel testing
            this.objKml = ds;
            //fi temp pel testing
            
            drawKmlDataSet(ds);
            
            
            
            
        } catch(Exception e) {
            Log.e("DirectionMap","Exception parsing kml. " +  e.getMessage());
            e.printStackTrace();
        }
		
	}

	public void drawKmlFromAsset(String asset) 
	{
		try 
		{
			InputStream in = getAssets().open(asset);
			drawKml(in);
		} catch (IOException e) 
		{
			Log.e( TAG , "Error reading asset " + e.getMessage() );
			e.printStackTrace();
		}
		

	}
	
	public void drawKmlFromFile(String filePath)
	{
		InputStream in = null;
		try 
		{
			File file = new File(filePath);
			in = new BufferedInputStream(new FileInputStream(file));
			this.firstChange = true;
			drawKml(in);
			if(in != null)
			{	
				in.close();
			}
		} catch (IOException e) 
		{
			Log.e( TAG , "Error reading asset " + e.getMessage() );
			e.printStackTrace();
		}
			
		
	}
	
	// button listener
	public void onFitListener(View v)
	{
		if(!gravant)
		{
			Toast.makeText(context, getText(R.string.alert_fit_sense_gravar).toString(), Toast.LENGTH_LONG).show();
			return;
		}
		synchronized(context)
		{
			Point mida = ScreenUtils.screenSize(context);
			
			int ref = (mida.x>mida.y)? mida.y : mida.x;
			
			int padding = Double.valueOf( ref*0.15 ).intValue();
		
			boolean firstPunt = true;
			if( objKml.getMarcadors().size() > 0 )
			{
				for(Iterator<Marcador> itMarc = objKml.getMarcadors().iterator(); itMarc.hasNext();)
				{
					Marcador marc = itMarc.next();
					
					Iterator<Punt> itPunt = marc.getPunts().iterator();
					
					if( itPunt.hasNext() )
					{
						Punt p = itPunt.next();
						if( firstPunt )
						{
							lat1 = p.getLatitud();
							lon1 = p.getLongitud();
							lat2 = p.getLatitud();
							lon2 = p.getLongitud();
							firstPunt = false;
						}else{
							if( p.getLatitud() < lat1 ) lat1 = p.getLatitud();
		        			if( p.getLatitud() > lat2 ) lat2 = p.getLatitud();
		        			if( p.getLongitud() < lon1 ) lon1 = p.getLongitud();
		        			if( p.getLongitud() > lon2 ) lon2 = p.getLongitud();
						}
						
						
					}
					
				}
				Marcador ruta = objKml.getMarcadorRuta();
				if( ruta != null )
				{
					Iterator<Punt> itPunt = ruta.getPunts().iterator();
					
					if( itPunt.hasNext() )
					{
						Punt p = itPunt.next();
						if( firstPunt )
						{
							lat1 = p.getLatitud();
							lon1 = p.getLongitud();
							lat2 = p.getLatitud();
							lon2 = p.getLongitud();
							firstPunt = false;
						}else{
							if( p.getLatitud() < lat1 ) lat1 = p.getLatitud();
		        			if( p.getLatitud() > lat2 ) lat2 = p.getLatitud();
		        			if( p.getLongitud() < lon1 ) lon1 = p.getLongitud();
		        			if( p.getLongitud() > lon2 ) lon2 = p.getLongitud();
						}
						
						
					}
				}
				
				mapa.animateCamera( CameraUpdateFactory.newLatLngBounds(  
						new LatLngBounds( 
								new LatLng( lat1, lon1  ),
								new LatLng( lat2, lon2  )
								),
						   padding )
						   );
			}
		}
	}
	
	//----------------------------
	//---------- GETTERS SETTERS
		/**
		 * @param loc
		 */
		public void setCurrentLocation(Location loc)
		{
			currentLocation = loc;
		}

		/**
		 * @return
		 */
		public Location getCurrentLocation()
		{
			return currentLocation;
		}
	
}
