package edu.uoc.dtorrens.dtblib.mapes;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;

import edu.uoc.dtorrens.dtblib.R;
import edu.uoc.dtorrens.dtblib.kml.KmlDataSet;
import edu.uoc.dtorrens.dtblib.kml.KmlHandler;
import edu.uoc.dtorrens.dtblib.kml.Marcador;
import edu.uoc.dtorrens.dtblib.kml.Punt;
import edu.uoc.dtorrens.dtblib.localitzacio.MapsDosLocation;
import edu.uoc.dtorrens.dtblib.mapes.overlay.MapItemizedOverlay;
import edu.uoc.dtorrens.dtblib.mapes.overlay.RouteOverlay;

public class BaseMapActivity extends MapActivity implements MapaInterface
{
	private static final String TAG = BaseMapActivity.class.getName();
	
	/**
	 */
	private MapView mapa;
	/**
	 */
	private MapController controladorMapa;
	
	/**
	 */
	private MyLocationOverlay overlayPosicio;	
	/**
	 */
	private List<Overlay> overlays;
	
	/**
	 */
	MapItemizedOverlay bgItemizedOverlay = null;
	//private SimpleLocationListener localitzador;
	/**
	 */
	KmlDataSet tempKml = null;	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.base_map);
		
		mapa = (MapView)findViewById(R.id.mapview);
		mapa.setBuiltInZoomControls(true);
		controladorMapa = mapa.getController();
		overlays = mapa.getOverlays();
		overlayPosicio = new MyLocationOverlay(this, mapa);
		
		//mapa.postInvalidate();
		
		
		//localitzador = new SimpleLocationListener(this);
		        
        drawKmlFromAsset("palomares.kml");
        
        /*StorageHelper sh = new StorageHelper(this);
        
        if(sh.setPublicDir("DtbTestMapa"))
        {
        	this.saveKml("exported.kml", sh.getPublicCurrentDirFile().getAbsolutePath());
        }*/
        
    }
		
	//---Copy&Paste
	

	public void onResume()
	{
		super.onResume();
		overlayPosicio.enableMyLocation();
		overlayPosicio.enableCompass();
		/*overlayPosicio.enableMyLocation();*/
		overlays.add(overlayPosicio);
		
		/*GeoPoint p = overlayPosicio.getMyLocation();
		if( p != null )
		{
			centrar(p);
		}else{
			Log.e(TAG,"My Location és null");
		}*/
	}
	
	public void onPause()
	{
		super.onPause();
		overlayPosicio.disableMyLocation();
		overlayPosicio.disableCompass();
		overlayPosicio.disableMyLocation();
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.activity_main_base_map, menu);
		return true;
	}

	@Override
	protected boolean isRouteDisplayed() {
		// TODO Auto-generated method stub
		return false;
	}

	public void centrar(GeoPoint centre)
	{
		controladorMapa.setCenter(centre);
		controladorMapa.setZoom(14);
	}
	
	public void centrar(int ilat, int ilon) 
	{
		GeoPoint centre = new GeoPoint(ilat, ilon);
		centrar(centre);
	}

	public void centrar(double dlat, double dlon) 
	{
		centrar((int)(dlat+1E6),(int)(dlon+1E6) );
	}

	/**
	 * Copy&Paste :: http://stackoverflow.com/questions/3109158/how-to-draw-a-path-on-a-map-using-kml-file/3109723#3109723
	 * Does the actual drawing of the route, based on the geo points provided in the nav set
	 *
	 * @param navSet     Navigation set bean that holds the route information, incl. geo pos
	 * @param color      Color in which to draw the lines
	 * @param mMapView01 Map view to draw onto
	 */
	public void drawPath(KmlDataSet kmlSet, int color, MapView mMapView01) {

	    Log.d(TAG, "map color before: " + color);        

	    // color correction for dining, make it darker
	    if (color == Color.parseColor("#add331")) color = Color.parseColor("#6C8715");
	    Log.d(TAG, "map color after: " + color);

	    Collection<Overlay> overlaysToAddAgain = new ArrayList<Overlay>();
	    
	    
	    if(mMapView01.getOverlays().size() > 0)
	    {	
		    for (Iterator<Overlay> iter = mMapView01.getOverlays().iterator(); iter.hasNext();) {
		        Overlay o = iter.next();
		        Log.d(TAG, "overlay type: " + o.getClass().getName());
		        if (!RouteOverlay.class.getName().equals(o.getClass().getName())) {
		            // mMapView01.getOverlays().remove(o);
		            overlaysToAddAgain.add(o);
		        }
		    }
		    
		    mMapView01.getOverlays().clear();
		    mMapView01.getOverlays().addAll(overlaysToAddAgain);
	    }
		    
	    if(kmlSet.getMarcadorRuta() != null)
	    {
		    String path = kmlSet.getMarcadorRuta().getCoordenades();
		    Log.d(TAG, "path=" + path);
		    if (path != null && path.trim().length() > 0) {
		        String[] pairs = path.trim().split(" ");
	
		        Log.d(TAG, "pairs.length=" + pairs.length);
	
		        String[] lngLat = pairs[0].split(","); // lngLat[0]=longitude lngLat[1]=latitude lngLat[2]=height
	
		        Log.d(TAG, "lnglat =" + lngLat + ", length: " + lngLat.length);
	
		        if (lngLat.length<3) lngLat = pairs[1].split(","); // if first pair is not transferred completely, take seconds pair //TODO 
	
		        try {
		            GeoPoint startGP = new GeoPoint((int) (Double.parseDouble(lngLat[1]) * 1E6), (int) (Double.parseDouble(lngLat[0]) * 1E6));
		            mMapView01.getOverlays().add(new RouteOverlay(startGP, startGP, 1));
		            GeoPoint gp1;
		            GeoPoint gp2 = startGP;
	
		            for (int i = 1; i < pairs.length; i++) // the last one would be crash
		            {
		                lngLat = pairs[i].split(",");
	
		                gp1 = gp2;
	
		                if (lngLat.length >= 2 && gp1.getLatitudeE6() != 0 && gp1.getLongitudeE6() != 0
		                        && gp2.getLatitudeE6() != 0 && gp2.getLongitudeE6() != 0) {
	
		                    // for GeoPoint, first:latitude, second:longitude
		                    gp2 = new GeoPoint((int) (Double.parseDouble(lngLat[1]) * 1E6), (int) (Double.parseDouble(lngLat[0]) * 1E6));
	
		                    if (gp2.getLatitudeE6() != 22200000) { 
		                        mMapView01.getOverlays().add(new RouteOverlay(gp1, gp2, 2, color));
		                        Log.d(TAG, "draw:" + gp1.getLatitudeE6() + "/" + gp1.getLongitudeE6() + " TO " + gp2.getLatitudeE6() + "/" + gp2.getLongitudeE6());
		                    }
		                }
		                // Log.d(myapp.APP,"pair:" + pairs[i]);
		            }
		            //routeOverlays.add(new RouteOverlay(gp2,gp2, 3));
		            mMapView01.getOverlays().add(new RouteOverlay(gp2, gp2, 3));
		        } catch (NumberFormatException e) {
		            Log.e(TAG, "Cannot draw route.", e);
		        }
		    }
	    }
	    // mMapView01.getOverlays().addAll(routeOverlays); // use the default color
	    mMapView01.setEnabled(true);
	}
	
	public void drawKmlFromAsset(String asset)
	{
		try 
		{
			InputStream in = getAssets().open(asset);
			drawKml(in);
		} catch (IOException e) 
		{
			Log.e( TAG , "Error reading asset " + e.getMessage() );
			e.printStackTrace();
		}
	}
	
	public void drawKml(InputStream in)
	{
		try{
        	
            KmlHandler kmlHandler = new KmlHandler();
            //get from file
           
            kmlHandler.parse(in);
            
            // get the results - should be a fully populated RSSFeed instance, or null on error
            KmlDataSet ds = kmlHandler.getKmlDataSet();

            //temp pel testing
            this.tempKml = ds;
            //fi temp pel testing
            
            // draw path
            //drawPath(ds, Color.parseColor("#add331"), mapa );
            drawPath(ds, Color.parseColor("#0000ff"), mapa );

            // find boundary by using itemized overlay
            GeoPoint destPoint = null;
            GeoPoint currentPoint = null;
            Punt ultimPunt = null;
            if (!ds.getMarcadors().isEmpty()) 
            {
            	Marcador ultimMarcador = ds.getMarcadors().get(ds.getMarcadors().size()-1);
            	if(!ultimMarcador.getPunts().isEmpty())
            	{
            		ultimPunt = ultimMarcador.getPunts().get(ultimMarcador.getPunts().size()-1);
            		destPoint = new GeoPoint((int)(ultimPunt.getLatitud()*1E6), (int)(ultimPunt.getLongitud()*1E6) );
            	}
            }
            
            // Copy&Pase
    		// Acquire a reference to the system Location Manager
            LocationManager locationManager = (LocationManager) this.getSystemService(Context.LOCATION_SERVICE);

            
            String locationProvider = LocationManager.NETWORK_PROVIDER;
            //String locationProvider = LocationManager.GPS_PROVIDER;
            Location lastKnownLocation = locationManager.getLastKnownLocation(locationProvider);
            
            //Drawable dot = this.getResources().getDrawable(R.drawable.ic_launcher);
			bgItemizedOverlay = new MapItemizedOverlay(getResources().getDrawable(R.drawable.ic_marker),this);
            
            if(lastKnownLocation != null && destPoint != null)
            {
	            currentPoint = new GeoPoint( Double.valueOf(lastKnownLocation.getLatitude()*1E6).intValue()
	                                                ,Double.valueOf(lastKnownLocation.getLongitude()*1E6).intValue() );
	
	            
	            Marcador marc = new Marcador();
	            marc.setCoordenada( lastKnownLocation.getLatitude(), lastKnownLocation.getLongitude() );
	            marc.setTitol("Posició actual");
	            marc.setDescripcio("Estàs aquí");
	            this.addMarcadorOverlay(marc);
	            
	            if(ultimPunt != null)
	            {
		            marc = new Marcador();
		            marc.setCoordenada( ultimPunt.getLatitud(), ultimPunt.getLongitud() );
		            marc.setTitol("Destinació");
		            marc.setDescripcio("Final de trajecte");
		            this.addMarcadorOverlay(marc);
	            }
	            //this.addItemOverlay(destPoint);
	            //this.addItemOverlay(currentPoint);
	            
	            // center and zoom in the map
	            Log.e(TAG,"PRE LOCATION");
	            controladorMapa.zoomToSpan(bgItemizedOverlay.getLatSpanE6()*2,bgItemizedOverlay.getLonSpanE6()*2);
	            controladorMapa.animateTo(new GeoPoint(
	                    (currentPoint.getLatitudeE6() + destPoint.getLatitudeE6()) / 2
	                    , (currentPoint.getLongitudeE6() + destPoint.getLongitudeE6()) / 2));
	            Log.e(TAG,"POST LOCATION");
            }
            
            //pintem tots els punts
            for( Iterator<Marcador> it = ds.getMarcadors().iterator(); it.hasNext(); )
            {
            	Marcador marc = it.next();
            	bgItemizedOverlay.addMarcador(marc);
            }
            mapa.getOverlays().add(bgItemizedOverlay);
            
        } catch(Exception e) {
            Log.e("DirectionMap","Exception parsing kml. " +  e.getMessage());
            e.printStackTrace();
        }
	}
	
	public void addMarcadorOverlay(Marcador marc)
	{
		if (this.bgItemizedOverlay == null)
		{
			Drawable dot = this.getResources().getDrawable(R.drawable.ic_marker);
			bgItemizedOverlay = new MapItemizedOverlay(dot,this);
			//mapa.getOverlays().add(bgItemizedOverlay);
			
		}
		bgItemizedOverlay.addMarcador(marc);
	}
	
	public void addItemOverlay(GeoPoint gpunt)
	{
		if (this.bgItemizedOverlay == null)
		{
			Drawable dot = this.getResources().getDrawable(R.drawable.ic_marker);
			bgItemizedOverlay = new MapItemizedOverlay(dot,this);
			//mapa.getOverlays().add(bgItemizedOverlay);
			
		}
		OverlayItem item = new OverlayItem(gpunt, null, null );
        bgItemizedOverlay.addOverlay(item);
	}
	
	public void saveKml(String fileName, String dir)
	{
		StringBuffer buffer = new StringBuffer();
		//creem el text
		buffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n");
		buffer.append("<Document>\n");
		buffer.append("\t<name> First exported KML </name>\n");
		buffer.append("\t<description> First exported KML </description>\n");
		
		// els punts
		for( Iterator<Marcador> it = this.tempKml.getMarcadors().iterator(); it.hasNext(); )
		{
			Marcador marc = it.next();
			buffer.append("\t\t<Placemark>\n");
			buffer.append("\t\t\t<name>" + marc.getTitol() + "</name>\n");
			buffer.append("\t\t\t<description>" + marc.getDescripcio() + "</description>\n");
			
			buffer.append("\t\t\t<Point>\n\t\t\t\t<coordinates>" + marc.getCoordenades() + "</coordinates>\n\t\t\t</Point>\n");
			
			buffer.append("\t\t</Placemark>\n");
		}
		
		//la ruta
		Marcador ruta = tempKml.getMarcadorRuta();
		if( ruta != null )
		{
			buffer.append("\t\t<Placemark>\n");
			buffer.append("\t\t\t<name>" + ruta.getTitol() + "</name>\n");
			buffer.append("\t\t\t<description>" + ruta.getDescripcio() + "</description>\n");
			
			buffer.append("\t\t\t<LineString>\n\t\t\t\t<coordinates>" + ruta.getCoordenades() + "</coordinates>\n\t\t\t</LineString>\n");
			
			buffer.append("\t\t</Placemark>");
		}
		
		buffer.append("</Document>\n</kml>\n");
		
		File file = new File(dir, fileName);
		try {
			FileWriter out = new FileWriter(file);
			out.write(buffer.toString());
			out.flush();
			out.close();
		} catch (IOException e) {
			Log.e(TAG, "ERROR FileWriter :: " + e.getMessage());
			e.printStackTrace();
		}
	}

	@Override
	public void onLocationChangeCallback(Location location, MapsDosLocation mapsLocation) {
		// TODO Auto-generated method stub
		
	}
	
}
