package edu.uoc.dtorrens.dtblib.mapes;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

import android.annotation.TargetApi;
import android.app.AlertDialog;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Point;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnCameraChangeListener;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptor;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.CameraPosition;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.LatLngBounds;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.android.gms.maps.model.PolylineOptions;
import com.google.android.maps.GeoPoint;

import edu.uoc.dtorrens.dtblib.R;
import edu.uoc.dtorrens.dtblib.kml.KmlDataSet;
import edu.uoc.dtorrens.dtblib.kml.KmlHandler;
import edu.uoc.dtorrens.dtblib.kml.Marcador;
import edu.uoc.dtorrens.dtblib.kml.Punt;
import edu.uoc.dtorrens.dtblib.localitzacio.MapsDosLocation;
import edu.uoc.dtorrens.utils.ScreenUtils;

public class BaseMapV2Activity extends AbstractBaseMap implements MapaInterface
{
	/**
	 */
	private final String TAG = BaseMapV2Activity.class.getName();
	
	
	
	/**
	 */
	protected Context context;
	
	/**
	 */
	protected GoogleMap mapa;
	/**
	 */
	private ArrayList<Marcador> marcadors = new ArrayList<Marcador>();
	
	/**
	 */
	KmlDataSet objKml = null;
	
	/**
	 */
	private boolean firstChange = true;
	/**
	 */
	private boolean boundsFound = false;
	/**
	 */
	double lat1;			// per enquadrar la vista



	/**
	 */
	double lat2;



	/**
	 */
	double lon1;



	/**
	 */
	double lon2;

	
	/**
	 */
	Location currentLocation = null;
	/**
	 */
	MapsDosLocation mapLocation = new MapsDosLocation(this);
	
	//new location v2
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.base_map_v2);
		
		this.context = this;
		
		mapLocation.setLocationManager( (LocationManager)getSystemService(LOCATION_SERVICE) );
	 	mapLocation.getCriteria().setAccuracy(Criteria.ACCURACY_FINE);
		
		init();
		
		
		drawKmlFromAsset("palomares.kml");
	}

	@TargetApi(Build.VERSION_CODES.GINGERBREAD)
	public void onResume()
	{
		super.onResume();
		setUpMapIfNeeded();
		
		mapLocation.getLocationManager().requestLocationUpdates(0L, 0.0f, mapLocation.getCriteria(), mapLocation, null);
	    mapa.setLocationSource(mapLocation);
	}
	public void onPause()
	{
		mapa.setLocationSource(null);
	    mapLocation.getLocationManager().removeUpdates(mapLocation);
		super.onPause();
	}
	
	//---------------------------
	// Inicialitzacions
	public void init()
	{
		setUpMapIfNeeded();
	}
	
	protected void setUpMapIfNeeded() {
        if (mapa == null) {
            mapa = ((SupportMapFragment) getSupportFragmentManager().findFragmentById(R.id.map))
                    .getMap();
            if (mapa != null) {
            	setUpMap();                
            }
        }
    }
	
	 protected void setUpMap() {
		 	mapa.getUiSettings().setZoomControlsEnabled(true);	   
		 	mapa.getUiSettings().setCompassEnabled(true);
		 	mapa.getUiSettings().setMyLocationButtonEnabled(true);
		 	mapa.setMyLocationEnabled(true);
		 	
		 	
		 	mapa.getMyLocation();
		 	
		 	mapa.setOnMarkerClickListener(new OnMarkerClickListener(){

				@Override
				public boolean onMarkerClick(Marker marker) {
					
					
					for( Iterator<Marcador> it = marcadors.iterator(); it.hasNext();  )
					{
						Marcador marc = it.next();
						
						if( marker.equals(marc.getMarker()) )
						{
							Log.i(TAG, "He trobat el marcador");
							AlertDialog.Builder dialog = new AlertDialog.Builder(context);
					    	dialog.setTitle(marc.getTitol() );
					    	dialog.setMessage(marc.getDescripcio());
					    	dialog.create();
					    	dialog.show();
					    	return true;
						}
					}
					
					return false;
				}
		 		
		 	});
		 	
		 	mapa.setOnCameraChangeListener(new OnCameraChangeListener(){

				@Override
				public void onCameraChange(CameraPosition cpos) {
					// TODO Auto-generated method stub
					Log.e(TAG, "Camera CHANGED !! ZOOM:" + cpos.zoom + " -- " + cpos.target.latitude + " : " + cpos.target.longitude);
					if(firstChange)
					{
						if(boundsFound)
						{
							Log.i(TAG, "BOUNDS FOUND");
							
							Point mida = ScreenUtils.screenSize(context);
							
							int ref = (mida.x>mida.y)? mida.y : mida.x;
							
							int padding = Double.valueOf( ref*0.15 ).intValue();
							
							mapa.animateCamera( CameraUpdateFactory.newLatLngBounds(  
									new LatLngBounds( 
											new LatLng( lat1, lon1  ),
											new LatLng( lat2, lon2  )
											),
									   padding ),
									   new GoogleMap.CancelableCallback() {
										
										@Override
										public void onFinish() {
											// TODO Auto-generated method stub
											firstChange = false;
										}
										
										@Override
										public void onCancel() {
											// TODO Auto-generated method stub
											firstChange = false;
										}
									});
						}else{
							Log.i(TAG, "BOUNDS NOT FOUND");
							mapa.animateCamera(CameraUpdateFactory.zoomTo(14),new GoogleMap.CancelableCallback() {
								
								@Override
								public void onFinish() {
									// TODO Auto-generated method stub
									firstChange = false;
								}
								
								@Override
								public void onCancel() {
									// TODO Auto-generated method stub
									firstChange = false;
								}
							});
						}
						//firstChange = false;
					}
				}});
	        
	    }
	
	public void onLocationChangeCallback(Location location, MapsDosLocation mapLocation)
	{
		currentLocation = location;
    	//he d'esperar a que el mapa existeixi
    	if(!firstChange)
    	{
	    	mapa.clear();
	    	
	    	//add marker with current location
	    	
	        
	    	drawKmlDataSet(objKml);
	    
	    	centrar(location.getLatitude(), location.getLongitude());
	    	
    	}
	}
	 
	@Override
	public void centrar(int ilat, int ilon) 
	{
		centrar( Double.valueOf( ilat )/1E6, Double.valueOf( ilon )/1E6  );
	}

	@Override
	public void centrar(double dlat, double dlon) {
		mapa.moveCamera(CameraUpdateFactory.newLatLng(  new LatLng( dlat, dlon ) ) );
	}

	@Override
	public void centrar(GeoPoint centre) 
	{
		centrar( centre.getLatitudeE6(), centre.getLongitudeE6() );
	}

	@Override
	public void addMarcadorOverlay(Marcador marc) 
	{
		addMarcadorOverlay(marc, ALTRES);
	}
	
	public void addMarcadorOverlay(Marcador marc, int tipus) 
	{
		Iterator<Punt> it = marc.getPunts().iterator();
		if( it.hasNext() )
		{
			float hue;
			
			if( tipus == DESTI )
			{
				Log.i(TAG,"COLOR DESTI");
				hue = BitmapDescriptorFactory.HUE_GREEN;
			}else if(tipus == ORIGEN){
				Log.i(TAG,"COLOR ORIGEN");
				hue = BitmapDescriptorFactory.HUE_RED;
			}else{
				Log.i(TAG,"COLOR ALTRES");
				hue = BitmapDescriptorFactory.HUE_ORANGE;
			}
					
			
			Punt p = it.next();
			Marker marker = mapa.addMarker(new MarkerOptions()
		       .position(new LatLng( p.getLatitud(), p.getLongitud() ) )
		       .title(marc.getTitol())
		       .icon(BitmapDescriptorFactory.defaultMarker(hue))
		       .snippet(marc.getDescripcio()));
			marc.setMarker(marker);
			marcadors.add(marc);
		}
		
		
	}
	
	public void drawPath(KmlDataSet ds, int color, GoogleMap mapa)
	{
		Marcador ruta = ds.getMarcadorRuta();
		if( ruta != null && ruta.getPunts().size()>0)
		{
			PolylineOptions polyOptions = new PolylineOptions();
			
			for( Iterator<Punt> it = ruta.getPunts().iterator(); it.hasNext(); )
			{
				Punt p = it.next();
				polyOptions.add( new LatLng( p.getLatitud(), p.getLongitud()  ) );
				
			}
			polyOptions.color(color);
			mapa.addPolyline(polyOptions);
		}
	}
	
	public void drawKmlDataSet(KmlDataSet ds)
	{
		// draw path
        //drawPath(ds, Color.parseColor("#add331"), mapa );
        drawPath(ds, Color.parseColor("#0000ff"), mapa );

        
        
        
        //pintem tots els punts
        Marcador marcOrigen = null;
        Marcador marcFinal = null;
        int count = 1;
        int mida = ds.getMarcadors().size();
        for( Iterator<Marcador> it = ds.getMarcadors().iterator(); it.hasNext(); )
        {
        	Marcador marc = it.next();
        	
        	if(marcOrigen == null)
        	{
        		marcOrigen = marc;
        		addMarcadorOverlay(marc, ORIGEN);
        	}else if(count == mida) {
        		marcFinal = marc;
        		addMarcadorOverlay(marc, DESTI);
        	}else{
        		addMarcadorOverlay(marc, ALTRES);
        	}
        	
        	count++;
        }
        
        if( marcOrigen != null && marcFinal != null )
        {
        	Punt pOrigen = null;
        	Punt pFinal = null;
        	
        	Iterator<Punt> itOrigen = marcOrigen.getPunts().iterator();
        	if(itOrigen.hasNext())
        	{
        		pOrigen = itOrigen.next();
        	}
        	
        	Iterator<Punt> itFinal = marcFinal.getPunts().iterator();
        	if(itFinal.hasNext())
        	{
        		pFinal = itFinal.next();
        	}
        	
        	if(pOrigen != null && pFinal != null)
        	{
        		Log.i(TAG, "ZOOM TO POINTS");
        		
        		
        		
        		if( pOrigen.getLatitud() < pFinal.getLatitud() )
        		{
        			lat1 = pOrigen.getLatitud();
        			lat2 = pFinal.getLatitud();
        		}else{
        			lat2 = pOrigen.getLatitud();
        			lat1 = pFinal.getLatitud();
        		}
        		
        		if( pOrigen.getLongitud() < pFinal.getLongitud() )
        		{
        			lon1 = pOrigen.getLongitud();
        			lon2 = pFinal.getLongitud();
        		}else{
        			lon2 = pOrigen.getLongitud();
        			lon1 = pFinal.getLongitud();
        		}
        		
        		if( currentLocation != null )
        		{
        			Log.i(TAG, "Current Location també juga a centrar");
        			if( currentLocation.getLatitude() < lat1 ) lat1 = currentLocation.getLatitude();
        			if( currentLocation.getLatitude() > lat2 ) lat2 = currentLocation.getLatitude();
        			if( currentLocation.getLongitude() < lon1 ) lon1 = currentLocation.getLongitude();
        			if( currentLocation.getLongitude() > lon2 ) lon2 = currentLocation.getLongitude();
        		}
        		
        		boundsFound = true;
            	
        	}
        }
	}
	
	@Override
	public void drawKml(InputStream in) {
		try{
        	
            KmlHandler kmlHandler = new KmlHandler();
            //get from file
           
            kmlHandler.parse(in);
            
            // get the results - should be a fully populated RSSFeed instance, or null on error
            KmlDataSet ds = kmlHandler.getKmlDataSet();

            //temp pel testing
            this.objKml = ds;
            //fi temp pel testing
            
            drawKmlDataSet(ds);
            
            
            
            
        } catch(Exception e) {
            Log.e("DirectionMap","Exception parsing kml. " +  e.getMessage());
            e.printStackTrace();
        }
		
	}

	public void drawKmlFromAsset(String asset) 
	{
		try 
		{
			InputStream in = getAssets().open(asset);
			drawKml(in);
		} catch (IOException e) 
		{
			Log.e( TAG , "Error reading asset " + e.getMessage() );
			e.printStackTrace();
		}
		

	}
	
	@Override
	public void saveKml(String fileName, String dir)
	{
		StringBuffer buffer = new StringBuffer();
		//creem el text
		buffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n");
		buffer.append("<Document>\n");
		buffer.append("\t<name> First exported KML </name>\n");
		buffer.append("\t<description> First exported KML </description>\n");
		
		// els punts
		for( Iterator<Marcador> it = this.objKml.getMarcadors().iterator(); it.hasNext(); )
		{
			Marcador marc = it.next();
			buffer.append("\t\t<Placemark>\n");
			buffer.append("\t\t\t<name>" + marc.getTitol() + "</name>\n");
			buffer.append("\t\t\t<description>" + marc.getDescripcio() + "</description>\n");
			
			buffer.append("\t\t\t<Point>\n\t\t\t\t<coordinates>" + marc.getCoordenades() + "</coordinates>\n\t\t\t</Point>\n");
			
			buffer.append("\t\t</Placemark>\n");
		}
		
		//la ruta
		Marcador ruta = objKml.getMarcadorRuta();
		if( ruta != null )
		{
			buffer.append("\t\t<Placemark>\n");
			buffer.append("\t\t\t<name>" + ruta.getTitol() + "</name>\n");
			buffer.append("\t\t\t<description>" + ruta.getDescripcio() + "</description>\n");
			
			buffer.append("\t\t\t<LineString>\n\t\t\t\t<coordinates>" + ruta.getCoordenades() + "</coordinates>\n\t\t\t</LineString>\n");
			
			buffer.append("\t\t</Placemark>");
		}
		
		buffer.append("</Document>\n</kml>\n");
		
		File file = new File(dir, fileName);
		try {
			FileWriter out = new FileWriter(file);
			out.write(buffer.toString());
			out.flush();
			out.close();
		} catch (IOException e) {
			Log.e(TAG, "ERROR FileWriter :: " + e.getMessage());
			e.printStackTrace();
		}
	}

	//---------- GETTERS SETTERS
	/**
	 * @param loc
	 */
	public void setCurrentLocation(Location loc)
	{
		currentLocation = loc;
	}

	/**
	 * @return
	 */
	public Location getCurrentLocation()
	{
		return currentLocation;
	}

	@Override
	public void onDialegReturn(Object[] result, int code) {
		// TODO Auto-generated method stub
		
	}
	

}
