package edu.uoc.dtorrens.dtblib.storage;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import android.content.Context;
import android.graphics.Bitmap;
import android.os.Environment;
import android.util.Log;

public class StorageHelper
{
	/**
	 */
	private final String TAG = StorageHelper.class.getName();
	
	private static final int BUFFER = 2048;
	
	public static final int STORAGE_PRIVATE = 0;
	public static final int STORAGE_PUBLIC = 1;
	
	/**
	 */
	private Context context;
	
	//utils
	/**
	 */
	private File privateBaseDir;
	/**
	 */
	private File privateCurrentDir;
	
	//public
	/**
	 */
	private File publicDir;
	/**
	 */
	boolean mExternalStorageAvailable = false;
	/**
	 */
	boolean mExternalStorageWriteable = false;
	/**
	 */
	boolean appDirAvailable = false;
	
	//filtre
	/**
	 */
	private FilenameFilter filtre;
	/**
	 */
	private List<String> extensions;
	
	//CONSTANTS
	public static final String tempDir = "tempkml"; 
	public static final String imageDir = "images"; 
	public static final String imageName = "image_";
	public static final String publicDirName = "Pasapas";
	
	public StorageHelper(Context ctx)
	{
		context = ctx;
		privateBaseDir = context.getFilesDir();
		privateCurrentDir = privateBaseDir;
		extensions = new LinkedList<String>();
		filtre = new FilenameFilter(){

			@Override
			public boolean accept(File dir, String filename) {
				
				if( extensions.size() == 0 )
				{
					return true;
				}
				
				Iterator<String> it = extensions.iterator();
				while(it.hasNext())
				{
					String ext = it.next();
					if( filename.endsWith(ext) )
					{
						return true;
					}
				}
				
				return false;
			}};
		
	}
	
	public void addFilter(String ext)
	{
		extensions.add(ext);
	}
	
	public void resetFilters()
	{
		extensions.clear();
	}
	
	//Private Storage
	public void createPrivateSubdir(String name)
	{
		Log.i(TAG,"Intentarem crear" + privateCurrentDir.getAbsolutePath().toString() + "/" + name);
		
		File f = new File(privateCurrentDir, name);
		f.mkdir();				
		privateCurrentDir = f;
	}
	
	//idem que l'anterior però mira que no hi sigui ja abans
	public void createPrivateSubdirNotExists(String name)
	{
		String current_name = privateCurrentDir.getName();
		Log.d(TAG," NAME OF CURRENT '" + current_name + "'");
		if( !current_name.equals(name) )
		{
			createPrivateSubdir(name);
		}
	}
	
	public void parentPrivateCurrentDir()
	{
		privateCurrentDir = new File(privateCurrentDir.getParent());
	}
	
	public void deleteDir(String path)
	{
		context.deleteFile(path);
	}
	
	public File[] getPrivateCurrentDirContents()
	{
		return privateCurrentDir.listFiles(filtre);
	}
	
	public String[] getPrivateCurrentDirContentsPaths()
	{
		File[] fa = getPrivateCurrentDirContents();
		String[] sa = new String[fa.length];		
		
		for(int i = 0; i < fa.length; i++)
		{
			sa[i] = fa[i].getAbsolutePath();
		}
		return sa;
	}
	
	public String[] getPrivateCurrentDirContentsNames()
	{
		File[] fa = getPrivateCurrentDirContents();
		String[] sa = new String[fa.length];		
		
		for(int i = 0; i < fa.length; i++)
		{
			sa[i] = fa[i].getName();
		}
		return sa;
	}
	
	public void createFilePrivateCurrentDir(String name, byte[] buffer) throws IOException
	{
		File f = new File(privateCurrentDir,name);
		FileOutputStream out = new FileOutputStream(f);
		out.write(buffer);
		out.close();
		
	}
	
	public void saveBitmapCurrentPrivateDir(Bitmap bmp, String name) throws FileNotFoundException
	{
		File file = new File(privateCurrentDir, name);
		OutputStream out = new FileOutputStream(file);
		bmp.compress(Bitmap.CompressFormat.PNG, 100, out);
		//byte[] data = 
	}
	
	public void deletePrivateCurrentDirContent()
	{
		borraRecursiu(privateCurrentDir);
	}
	
	public void deletePrivateBaseDirContent()
	{
		borraRecursiuContingut(privateBaseDir);
	}
	
	public void currentPrivateToRoot()
	{
		privateCurrentDir = privateBaseDir;
	}
	
	//Generals	
	/**
	 * Borra el contingut d'un directory, el directori també
	 */
	public void borraRecursiu(File d)
	{
		if(d.isDirectory())
		{
			for(File fill : d.listFiles())
			{
				borraRecursiu(fill);
			}
		}
		d.delete();
		
	}
	
	/**
	 * Borra tot el contingut d'un directori, però el directory NO
	 * @param d
	 */
	public void borraRecursiuContingut(File d)
	{
		if(d.isDirectory())
		{
			for(File fill : d.listFiles())
			{
				borraRecursiu(fill);
			}
		}
	}
	
	//torna els arxius de l'arrel de l'emmagatzematge privat
	public String[] getPrivateFileList()
	{
		return context.fileList();
	}
	
	//mètodes útils
	
	//--------------------------------------------------------------------------------
	// Algoritmes de compressio i descompressó de 
	
	// Comprssio amb subdirs ----------------------------------------------------------
	// from :: http://www.devx.com/tips/Tip/14049
	public void zipDir(String dir2zip, ZipOutputStream zos) 
	{ 
	    try 
	   { 
	        //create a new File object based on the directory we 
	        //have to zip File    
	        File zipDir = new File(dir2zip); 
	        //get a listing of the directory content 
	        String[] dirList = zipDir.list(); 
	        byte[] readBuffer = new byte[2156]; 
	        int bytesIn = 0; 
	        //loop through dirList, and zip the files 
	        for(int i=0; i<dirList.length; i++) 
	        { 
	            File f = new File(zipDir, dirList[i]); 
		        if(f.isDirectory()) 
		        { 
		                //if the File object is a directory, call this 
		                //function again to add its content recursively 
		            String filePath = f.getPath(); 
		            zipDir(filePath, zos); 
		                //loop again 
		            continue; 
		        } 
		            //if we reached here, the File object f was not 
		            //a directory 
		            //create a FileInputStream on top of f 
		            FileInputStream fis = new FileInputStream(f); 
		            //create a new zip entry 
		            String inZipPath = f.getPath().replace(this.getPrivateCurrentDir() + "/", "");
		            ZipEntry anEntry = new ZipEntry(inZipPath); 
		            //place the zip entry in the ZipOutputStream object 
		            zos.putNextEntry(anEntry); 
		            //now write the content of the file to the ZipOutputStream 
		            while((bytesIn = fis.read(readBuffer)) != -1) 
		            { 
		                zos.write(readBuffer, 0, bytesIn); 
		            } 
		           //close the Stream 
		           fis.close(); 
		    } 
		} 
		catch(Exception e) 
		{ 
		    e.printStackTrace();
		} 
	}
	
	public boolean saveToZipDir(String fileName)
	{
		try 
		{ 
			setPublicDir( StorageHelper.publicDirName );
		    // create a ZipOutputStream to zip the data to 
		    ZipOutputStream zos = new 
		           ZipOutputStream(new FileOutputStream(publicDir.getAbsolutePath() + "/" + fileName)); 
		    // assuming that there is a directory named inFolder (If there 
		    // isn't create one) in the same directory as the one the code 
		    // runs from, 
		    // call the zipDir method 
		    zipDir( this.getPrivateBaseDir() + "/" + StorageHelper.tempDir , zos); 
		    // close the stream 
		    zos.close(); 
		} 
		catch(Exception e) 
		{ 
		    e.printStackTrace();
		    return false;
		}
		return true;
	}
	
	// descompressió amb subdirs
	// http://examples.javacodegeeks.com/core-java/util/zip/extract-zip-file-with-subdirectories/
	public void saveToDirZip(String filename) {
		
		
		File srcFile = new File(filename);
		
		// create a directory with the same name to which the contents will be extracted
		String zipPath = this.getPrivateCurrentDir();
		File temp = new File(zipPath);
		temp.mkdir();
		
		ZipFile zipFile = null;
		
		try {
			
			zipFile = new ZipFile(srcFile);
			
			// get an enumeration of the ZIP file entries
			Enumeration<? extends ZipEntry> e = zipFile.entries();
			
			while (e.hasMoreElements()) {
				
				ZipEntry entry = e.nextElement();
				
				File destinationPath = new File(zipPath, entry.getName());
				 
				//create parent directories
				destinationPath.getParentFile().mkdirs();
				
				// if the entry is a file extract it
				if (entry.isDirectory()) {
					continue;
				}
				else {
					
					System.out.println("Extracting file: " + destinationPath);
					
					BufferedInputStream bis = new BufferedInputStream(zipFile.getInputStream(entry));

					int b;
					byte buffer[] = new byte[1024];

					FileOutputStream fos = new FileOutputStream(destinationPath);
					
					BufferedOutputStream bos = new BufferedOutputStream(fos, 1024);

					while ((b = bis.read(buffer, 0, 1024)) != -1) {
						bos.write(buffer, 0, b);
					}
					
					bos.close();
					bis.close();
					
				}
				
			}
			
		}
		catch (IOException ioe) {
			System.out.println("Error opening zip file" + ioe);
		}
		 finally {
			 try {
				 if (zipFile!=null) {
					 zipFile.close();
				 }
			 }
			 catch (IOException ioe) {
					System.out.println("Error while closing zip file" + ioe);
			 }
		 }
		
	}
	
	
	// SENSE SUBDIRS ----------------------------------------------------
	// http://www.jondev.net/
	//al directori public
	public boolean saveToZip(String fileName) throws Exception
	{
		File[] files = getPrivateCurrentDirContents();
		if( files.length > 0 )
		{
			try 
			{
				BufferedInputStream origin = null;
				Log.i(TAG, "ZIP NAME :: " + publicDir.getAbsolutePath() + "/" + fileName);
				FileOutputStream dest = new FileOutputStream(publicDir.getAbsolutePath() + "/" + fileName);
				ZipOutputStream out = new ZipOutputStream( new BufferedOutputStream(dest) );
				
				byte[] data = new byte[BUFFER];
				
				for(int i = 0; i < files.length; i++)
				{
					Log.i(TAG, "ZIPING Afegint :: " + files[i]);
					FileInputStream fi = new FileInputStream(files[i]);
					origin = new BufferedInputStream(fi, BUFFER);
					ZipEntry entry = new ZipEntry(files[i].getName()); 
				    out.putNextEntry(entry); 
				    int count; 
				    while ((count = origin.read(data, 0, BUFFER)) != -1) 
				    { 
				          out.write(data, 0, count); 
				    } 
				    origin.close(); 
				}
				out.close();
				
			} catch (Exception e) 
			{
				e.printStackTrace();
				throw new Exception(e);
			}
			
		}else
		{
			return false;
		}
		
		return true;
	}
	
	//al directory privat
	public void zipToDir(String zipFile)
	{
		String location = privateCurrentDir.getAbsolutePath() + "/";
		//ULL el directory de destí ha d'existir !!!
		try  { 
		      FileInputStream fin = new FileInputStream(zipFile); 
		      ZipInputStream zin = new ZipInputStream(fin); 
		      ZipEntry ze = null; 
		      while ((ze = zin.getNextEntry()) != null) { 
		        Log.v("Decompress", "Unzipping " + ze.getName()); 		 
		        if(ze.isDirectory()) { 
		          dirChecker(ze.getName()); 
		        } else { 
		          FileOutputStream fout = new FileOutputStream( location + ze.getName()); 
		          for (int c = zin.read(); c != -1; c = zin.read()) { 
		            fout.write(c); 
		          } 
		 
		          zin.closeEntry(); 
		          fout.close(); 
		        } 
		         
		      } 
		      zin.close(); 
		    } catch(Exception e) { 
		      Log.e("Decompress", "unzip", e); 
		    } 
	}
	
	private void dirChecker(String dir)
	{
		File f = new File(privateCurrentDir.getAbsolutePath() + "/" + dir);
		if(!f.isDirectory()) {
			f.mkdirs();
		}
	}
	
	//-----------------------------------------------
	// a sd card
	public boolean setPublicDir(String dirName)
	{
		if(!checkExternalState())
		{
			Log.e(TAG,"EXTERNAL STORAGE NO ACESSIBLE");
			return false;
		}
		
		//getFilesDir() ?
		//Log.d(TAG,"DIRECTORY " + Environment.getExternalStorageDirectory() );
		Log.d(TAG,"DIRECTORY " + " /mnt/sdcard/" );
		
		String rutaDir = "/mnt/sdcard/" + dirName;
		
		File dir = new File(rutaDir);
		if(dir.exists() && dir.isDirectory()) {
		    // do something here
			Log.d(TAG, "El dir SI existeix");
			publicDir = dir;
		}else{
			Log.d(TAG, "El dir NO existeix");
			//creem el dir
			if(dir.mkdir())
			{
				Log.d(TAG, "Directory creat :: " + dir.getAbsolutePath());
			}else{
				Log.e(TAG, " ERROR Directory NO creat :: " + dir.getAbsolutePath());
				return false;
			}
			
		}			
		return true;
	}
	
	public boolean checkExternalState()
	{
		String state = Environment.getExternalStorageState();

		if (Environment.MEDIA_MOUNTED.equals(state)) {
		    // We can read and write the media
			Log.d(TAG,"EXTERNAL STORAGE DISPONIBLE R/W");
		    mExternalStorageAvailable = mExternalStorageWriteable = true;
		} else if (Environment.MEDIA_MOUNTED_READ_ONLY.equals(state)) {
		    // We can only read the media
		    mExternalStorageAvailable = true;
		    mExternalStorageWriteable = false;
		    Log.e(TAG,"EXTERNAL STORAGE DISPONIBLE ONLY R");
		} else {
		    // Something else is wrong. It may be one of many other states, but all we need
		    //  to know is we can neither read nor write
		    mExternalStorageAvailable = mExternalStorageWriteable = false;
		    Log.e(TAG,"EXTERNAL STORAGE NO DISPONIBLE");
		}
		
		return  mExternalStorageAvailable;
		
	}
	
	public File[] getPublicCurrentDirContents()
	{
		return publicDir.listFiles(filtre);
	}
	
	public String[] getPublicCurrentDirContentsPaths()
	{
		File[] fa = getPublicCurrentDirContents();
		String[] sa = new String[fa.length];		
		
		for(int i = 0; i < fa.length; i++)
		{
			sa[i] = fa[i].getAbsolutePath();
		}
		return sa;
	}
	
	public String[] getPublicCurrentDirContentsNames()
	{
		File[] fa = getPublicCurrentDirContents();
		String[] sa = new String[fa.length];		
		
		for(int i = 0; i < fa.length; i++)
		{
			sa[i] = fa[i].getName();
		}
		return sa;
	}
	
	//-------------------
	// GETTEERS
	
	public File getPrivateCurrentDirFile()
	{
		return this.privateCurrentDir;
	}
	
	public File getPublicCurrentDirFile()
	{
		return this.publicDir;
	}
	
	public String getPrivateBaseDir()
	{
		return privateBaseDir.getAbsolutePath();
	}
	public String getPrivateCurrentDir()
	{
		return privateCurrentDir.toString();
	}
}
